@@ -1,6 +1,6 @@
 //! # ndrustfft: *n*-dimensional complex-to-complex FFT, real-to-complex FFT and real-to-real DCT
 //!
-//! This library is a wrapper for RustFFT that enables performing FFTs of complex-, real-valued
+//! This library is a wrapper for `RustFFT` that enables performing FFTs of complex-, real-valued
 //! data and DCT's on *n*-dimensional arrays (ndarray).
 //!
 //! ndrustfft provides Handler structs for FFT's and DCTs, which must be provided
@@ -20,12 +20,12 @@
 //! ## Example
 //! 2-Dimensional real-to-complex fft along first axis
 //! ```
-//! use ndarray::{Array, Dim, Ix};
+//! use ndarray::{Array2, Dim, Ix};
 //! use ndrustfft::{ndrfft, Complex, FftHandler};
 //!
 //! let (nx, ny) = (6, 4);
-//! let mut data = Array::<f64, Dim<[Ix; 2]>>::zeros((nx, ny));
-//! let mut vhat = Array::<Complex<f64>, Dim<[Ix; 2]>>::zeros((nx / 2 + 1, ny));
+//! let mut data = Array2::<f64>::zeros((nx, ny));
+//! let mut vhat = Array2::<Complex<f64>>::zeros((nx / 2 + 1, ny));
 //! for (i, v) in data.iter_mut().enumerate() {
 //!     *v = i as f64;
 //! }
@@ -149,12 +149,12 @@ macro_rules! create_transform_par {
 /// # Example
 /// 2-Dimensional real-to-complex fft along first axis
 /// ```
-/// use ndarray::{Array, Dim, Ix};
+/// use ndarray::{Array2, Dim, Ix};
 /// use ndrustfft::{ndrfft, Complex, FftHandler};
 ///
 /// let (nx, ny) = (6, 4);
-/// let mut data = Array::<f64, Dim<[Ix; 2]>>::zeros((nx, ny));
-/// let mut vhat = Array::<Complex<f64>, Dim<[Ix; 2]>>::zeros((nx / 2 + 1, ny));
+/// let mut data = Array2::<f64>::zeros((nx, ny));
+/// let mut vhat = Array2::<Complex<f64>>::zeros((nx / 2 + 1, ny));
 /// for (i, v) in data.iter_mut().enumerate() {
 ///     *v = i as f64;
 /// }
@@ -166,6 +166,7 @@ macro_rules! create_transform_par {
 ///     0,
 /// );
 /// ```
+#[derive(Clone)]
 pub struct FftHandler<T> {
     n: usize,
     m: usize,
@@ -175,7 +176,7 @@ pub struct FftHandler<T> {
 }
 
 impl<T: FftNum> FftHandler<T> {
-    /// Creates a new FftHandler.
+    /// Creates a new `FftHandler`.
     ///
     /// # Arguments
     ///
@@ -189,6 +190,8 @@ impl<T: FftNum> FftHandler<T> {
     /// use ndrustfft::FftHandler;
     /// let handler: FftHandler<f64> = FftHandler::new(10);
     /// ```
+    #[allow(clippy::similar_names)]
+    #[must_use]
     pub fn new(n: usize) -> Self {
         let mut planner = FftPlanner::<T>::new();
         let fwd = planner.plan_fft_forward(n);
@@ -204,17 +207,18 @@ impl<T: FftNum> FftHandler<T> {
     }
 
     fn fft_lane(&self, data: &[Complex<T>], out: &mut [Complex<T>]) {
-        self.assert_size(self.n, data.len());
-        self.assert_size(self.n, out.len());
+        Self::assert_size(self.n, data.len());
+        Self::assert_size(self.n, out.len());
         for (b, d) in out.iter_mut().zip(data.iter()) {
             *b = *d;
         }
         self.plan_fwd.process(out);
     }
 
+    #[allow(clippy::cast_precision_loss)]
     fn ifft_lane(&self, data: &[Complex<T>], out: &mut [Complex<T>]) {
-        self.assert_size(self.n, data.len());
-        self.assert_size(self.n, out.len());
+        Self::assert_size(self.n, data.len());
+        Self::assert_size(self.n, out.len());
         for (b, d) in out.iter_mut().zip(data.iter()) {
             *b = *d;
         }
@@ -226,33 +230,34 @@ impl<T: FftNum> FftHandler<T> {
     }
 
     fn rfft_lane(&mut self, data: &[T], out: &mut [Complex<T>]) {
-        self.assert_size(self.n, data.len());
-        self.assert_size(self.m, out.len());
+        Self::assert_size(self.n, data.len());
+        Self::assert_size(self.m, out.len());
         for (b, d) in self.buffer.iter_mut().zip(data.iter()) {
             *b = Complex::new(*d, T::zero());
         }
         self.plan_fwd.process(&mut self.buffer);
-        for (b, d) in self.buffer[0..self.n / 2 + 1].iter().zip(out.iter_mut()) {
+        for (b, d) in self.buffer[0..=self.n / 2].iter().zip(out.iter_mut()) {
             *d = *b;
         }
     }
 
     fn rfft_lane_par(&self, data: &[T], out: &mut [Complex<T>]) {
-        self.assert_size(self.n, data.len());
-        self.assert_size(self.m, out.len());
+        Self::assert_size(self.n, data.len());
+        Self::assert_size(self.m, out.len());
         let mut buffer = vec![Complex::zero(); self.n];
         for (b, d) in buffer.iter_mut().zip(data.iter()) {
             *b = Complex::new(*d, T::zero());
         }
         self.plan_fwd.process(&mut buffer);
-        for (b, d) in buffer[0..self.n / 2 + 1].iter().zip(out.iter_mut()) {
+        for (b, d) in buffer[0..=self.n / 2].iter().zip(out.iter_mut()) {
             *d = *b;
         }
     }
 
+    #[allow(clippy::cast_precision_loss)]
     fn irfft_lane(&mut self, data: &[Complex<T>], out: &mut [T]) {
-        self.assert_size(self.m, data.len());
-        self.assert_size(self.n, out.len());
+        Self::assert_size(self.m, data.len());
+        Self::assert_size(self.n, out.len());
         let m = data.len();
         for (b, d) in self.buffer[..m].iter_mut().zip(data.iter()) {
             *b = *d;
@@ -268,8 +273,9 @@ impl<T: FftNum> FftHandler<T> {
         }
     }
 
+    #[allow(clippy::cast_precision_loss)]
     fn irfft_lane_par(&self, data: &[Complex<T>], out: &mut [T]) {
-        self.assert_size(self.m, data.len());
+        Self::assert_size(self.m, data.len());
         let m = data.len();
         let mut buffer = vec![Complex::zero(); self.n];
         for (b, d) in buffer[..m].iter_mut().zip(data.iter()) {
@@ -286,7 +292,7 @@ impl<T: FftNum> FftHandler<T> {
         }
     }
 
-    fn assert_size(&self, n: usize, size: usize) {
+    fn assert_size(n: usize, size: usize) {
         assert!(
             n == size,
             "Size mismatch in fft, got {} expected {}",
@@ -300,12 +306,12 @@ create_transform!(
     /// Complex-to-complex Fourier Transform (serial).
     /// # Example
     /// ```
-    /// use ndarray::{Array, Dim, Ix};
+    /// use ndarray::{Array2, Dim, Ix};
     /// use ndrustfft::{ndfft, Complex, FftHandler};
     ///
     /// let (nx, ny) = (6, 4);
-    /// let mut data = Array::<Complex<f64>, Dim<[Ix; 2]>>::zeros((nx, ny));
-    /// let mut vhat = Array::<Complex<f64>, Dim<[Ix; 2]>>::zeros((nx, ny));
+    /// let mut data = Array2::<Complex<f64>>::zeros((nx, ny));
+    /// let mut vhat = Array2::<Complex<f64>>::zeros((nx, ny));
     /// for (i, v) in data.iter_mut().enumerate() {
     ///     v.re = i as f64;
     ///     v.im = -1.0*i as f64;
@@ -324,12 +330,12 @@ create_transform!(
     /// Complex-to-complex Inverse Fourier Transform (serial).
     /// # Example
     /// ```
-    /// use ndarray::{Array, Dim, Ix};
+    /// use ndarray::Array2;
     /// use ndrustfft::{ndfft, ndifft, Complex, FftHandler};
     ///
     /// let (nx, ny) = (6, 4);
-    /// let mut data = Array::<Complex<f64>, Dim<[Ix; 2]>>::zeros((nx, ny));
-    /// let mut vhat = Array::<Complex<f64>, Dim<[Ix; 2]>>::zeros((nx, ny));
+    /// let mut data = Array2::<Complex<f64>>::zeros((nx, ny));
+    /// let mut vhat = Array2::<Complex<f64>>::zeros((nx, ny));
     /// for (i, v) in data.iter_mut().enumerate() {
     ///     v.re = i as f64;
     ///     v.im = -1.0*i as f64;
@@ -349,12 +355,12 @@ create_transform!(
     /// Real-to-complex Fourier Transform (serial).
     /// # Example
     /// ```
-    /// use ndarray::{Array, Dim, Ix};
+    /// use ndarray::Array2;
     /// use ndrustfft::{ndrfft, Complex, FftHandler};
     ///
     /// let (nx, ny) = (6, 4);
-    /// let mut data = Array::<f64, Dim<[Ix; 2]>>::zeros((nx, ny));
-    /// let mut vhat = Array::<Complex<f64>, Dim<[Ix; 2]>>::zeros((nx / 2 + 1, ny));
+    /// let mut data = Array2::<f64>::zeros((nx, ny));
+    /// let mut vhat = Array2::<Complex<f64>>::zeros((nx / 2 + 1, ny));
     /// for (i, v) in data.iter_mut().enumerate() {
     ///     *v = i as f64;
     /// }
@@ -377,12 +383,12 @@ create_transform!(
     /// Complex-to-real inverse Fourier Transform (serial).
     /// # Example
     /// ```
-    /// use ndarray::{Array, Dim, Ix};
+    /// use ndarray::Array2;
     /// use ndrustfft::{ndirfft, Complex, FftHandler};
     ///
     /// let (nx, ny) = (6, 4);
-    /// let mut data = Array::<f64, Dim<[Ix; 2]>>::zeros((nx, ny));
-    /// let mut vhat = Array::<Complex<f64>, Dim<[Ix; 2]>>::zeros((nx / 2 + 1, ny));
+    /// let mut data = Array2::<f64>::zeros((nx, ny));
+    /// let mut vhat = Array2::<Complex<f64>>::zeros((nx / 2 + 1, ny));
     /// for (i, v) in vhat.iter_mut().enumerate() {
     ///     v.re = i as f64;
     /// }
@@ -460,12 +466,12 @@ create_transform_par!(
 /// # Example
 /// 2-Dimensional real-to-real dft along second axis
 /// ```
-/// use ndarray::{Array, Dim, Ix};
+/// use ndarray::Array2;
 /// use ndrustfft::{DctHandler, nddct1};
 ///
 /// let (nx, ny) = (6, 4);
-/// let mut data = Array::<f64, Dim<[Ix; 2]>>::zeros((nx, ny));
-/// let mut vhat = Array::<f64, Dim<[Ix; 2]>>::zeros((nx, ny));
+/// let mut data = Array2::<f64>::zeros((nx, ny));
+/// let mut vhat = Array2::<f64>::zeros((nx, ny));
 /// for (i, v) in data.iter_mut().enumerate() {
 ///     *v = i as f64;
 /// }
@@ -477,6 +483,7 @@ create_transform_par!(
 ///     1,
 /// );
 /// ```
+#[derive(Clone)]
 pub struct DctHandler<T> {
     n: usize,
     plan: Arc<dyn rustfft::Fft<T>>,
@@ -484,7 +491,7 @@ pub struct DctHandler<T> {
 }
 
 impl<T: FftNum> DctHandler<T> {
-    /// Creates a new DctHandler.
+    /// Creates a new `DctHandler`.
     ///
     /// # Arguments
     ///
@@ -497,6 +504,7 @@ impl<T: FftNum> DctHandler<T> {
     /// use ndrustfft::DctHandler;
     /// let handler: DctHandler<f64> = DctHandler::new(10);
     /// ```
+    #[must_use]
     pub fn new(n: usize) -> Self {
         let m = 2 * (n - 1);
         let mut planner = FftPlanner::<T>::new();
@@ -521,7 +529,7 @@ impl<T: FftNum> DctHandler<T> {
     fn dct1_lane(&mut self, data: &[T], out: &mut [T]) {
         self.assert_size(data.len());
         let m = self.buffer.len();
-        for b in self.buffer.iter_mut() {
+        for b in &mut self.buffer.iter_mut() {
             b.re = T::zero();
             b.im = T::zero();
         }
@@ -568,12 +576,12 @@ create_transform!(
     ///
     /// # Example
     /// ```
-    /// use ndarray::{Array, Dim, Ix};
+    /// use ndarray::Array2;
     /// use ndrustfft::{DctHandler, nddct1};
     ///
     /// let (nx, ny) = (6, 4);
-    /// let mut data = Array::<f64, Dim<[Ix; 2]>>::zeros((nx, ny));
-    /// let mut vhat = Array::<f64, Dim<[Ix; 2]>>::zeros((nx, ny));
+    /// let mut data = Array2::<f64>::zeros((nx, ny));
+    /// let mut vhat = Array2::<f64>::zeros((nx, ny));
     /// for (i, v) in data.iter_mut().enumerate() {
     ///     *v = i as f64;
     /// }
@@ -607,14 +615,14 @@ create_transform!(
 #[cfg(test)]
 mod test {
     use super::*;
-    use ndarray::{Array, Dim, Ix};
+    use ndarray::Array2;
 
     #[test]
     /// Successive forward and inverse transform
     fn test_fft() {
         let (nx, ny) = (6, 4);
-        let mut data = Array::<Complex<f64>, Dim<[Ix; 2]>>::zeros((nx, ny));
-        let mut vhat = Array::<Complex<f64>, Dim<[Ix; 2]>>::zeros((nx, ny));
+        let mut data = Array2::<Complex<f64>>::zeros((nx, ny));
+        let mut vhat = Array2::<Complex<f64>>::zeros((nx, ny));
         for (i, v) in data.iter_mut().enumerate() {
             v.re = i as f64;
             v.im = -1.0 * i as f64;
@@ -640,9 +648,9 @@ mod test {
     /// Successive forward and inverse transform
     fn test_dct_serial_vs_parallel() {
         let (nx, ny) = (6, 4);
-        let mut data = Array::<f64, Dim<[Ix; 2]>>::zeros((nx, ny));
-        let mut vhat = Array::<f64, Dim<[Ix; 2]>>::zeros((nx, ny));
-        let mut vhat_par = Array::<f64, Dim<[Ix; 2]>>::zeros((nx, ny));
+        let mut data = Array2::<f64>::zeros((nx, ny));
+        let mut vhat = Array2::<f64>::zeros((nx, ny));
+        let mut vhat_par = Array2::<f64>::zeros((nx, ny));
         for (i, v) in data.iter_mut().enumerate() {
             *v = i as f64;
         }
@@ -663,8 +671,8 @@ mod test {
     /// Successive forward and inverse transform
     fn test_rfft() {
         let (nx, ny) = (6, 4);
-        let mut data = Array::<f64, Dim<[Ix; 2]>>::zeros((nx, ny));
-        let mut vhat = Array::<Complex<f64>, Dim<[Ix; 2]>>::zeros((nx, ny / 2 + 1));
+        let mut data = Array2::<f64>::zeros((nx, ny));
+        let mut vhat = Array2::<Complex<f64>>::zeros((nx, ny / 2 + 1));
         for (i, v) in data.iter_mut().enumerate() {
             *v = i as f64;
         }
@@ -686,9 +694,9 @@ mod test {
     /// Successive forward and inverse transform
     fn test_rfft_serial_vs_parallel() {
         let (nx, ny) = (6, 4);
-        let mut data = Array::<f64, Dim<[Ix; 2]>>::zeros((nx, ny));
-        let mut vhat = Array::<Complex<f64>, Dim<[Ix; 2]>>::zeros((nx, ny / 2 + 1));
-        let mut vhat_par = Array::<Complex<f64>, Dim<[Ix; 2]>>::zeros((nx, ny / 2 + 1));
+        let mut data = Array2::<f64>::zeros((nx, ny));
+        let mut vhat = Array2::<Complex<f64>>::zeros((nx, ny / 2 + 1));
+        let mut vhat_par = Array2::<Complex<f64>>::zeros((nx, ny / 2 + 1));
         for (i, v) in data.iter_mut().enumerate() {
             *v = i as f64;
         }
